
// DO NOT EDIT THIS FILE - it is machine generated -*- c++ -*-

#ifndef __javax_swing_text_Position$Bias__
#define __javax_swing_text_Position$Bias__

#pragma interface

#include <java/lang/Object.h>
extern "Java"
{
  namespace javax
  {
    namespace swing
    {
      namespace text
      {
          class Position$Bias;
      }
    }
  }
}

class javax::swing::text::Position$Bias : public ::java::lang::Object
{

  Position$Bias(::java::lang::String *);
public:
  ::java::lang::String * toString();
  static ::javax::swing::text::Position$Bias * Backward;
  static ::javax::swing::text::Position$Bias * Forward;
private:
  ::java::lang::String * __attribute__((aligned(__alignof__( ::java::lang::Object)))) name;
public:
  static ::java::lang::Class class$;
};

#endif // __javax_swing_text_Position$Bias__
